/*
 *  arch-tag: Implementation of generic audio player source object
 *
 *  Copyright (C) 2004 James Livingston  <jrl@ids.org.au>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#if defined(HAVE_HAL_0_5) || defined(HAVE_HAL_0_2)
#define HAVE_HAL 1
#endif


#include <gtk/gtktreeview.h>
#include <string.h>
#include "rhythmdb.h"
#include <libgnome/gnome-i18n.h>
#ifdef HAVE_HAL
#include <libhal.h>
#include <dbus/dbus.h>
#endif
#include <libgnomevfs/gnome-vfs-volume.h>
#include <libgnomevfs/gnome-vfs-volume-monitor.h>
#include "eel-gconf-extensions.h"
#include "rb-generic-player-source.h"
#include "rb-debug.h"
#include "rb-util.h"
#include "rhythmdb.h"

static GObject *rb_generic_player_source_constructor (GType type, guint n_construct_properties,
						      GObjectConstructParam *construct_properties);
static void rb_generic_player_source_dispose (GObject *object);

static gboolean impl_show_popup (RBSource *source);
static void rb_generic_player_load_songs (RBGenericPlayerSource *source);
static gchar *rb_generic_player_get_mount_path (GnomeVFSVolume *volume);

typedef struct
{
	char *mount_path;
} RBGenericPlayerSourcePrivate;


G_DEFINE_TYPE (RBGenericPlayerSource, rb_generic_player_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
#define GENERIC_PLAYER_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_GENERIC_PLAYER_SOURCE, RBGenericPlayerSourcePrivate))

typedef const char * (*RBSourceStringFunc) (RBSource *source);

static void
rb_generic_player_source_class_init (RBGenericPlayerSourceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);

	object_class->constructor = rb_generic_player_source_constructor;
	object_class->dispose = rb_generic_player_source_dispose;

	source_class->impl_show_popup = impl_show_popup;

	g_type_class_add_private (klass, sizeof (RBGenericPlayerSourcePrivate));
}

static void
rb_generic_player_source_init (RBGenericPlayerSource *source)
{

}

static GObject *
rb_generic_player_source_constructor (GType type, guint n_construct_properties,
			       GObjectConstructParam *construct_properties)
{
	GObjectClass *klass, *parent_class; 
	RBGenericPlayerSource *source; 

	klass = G_OBJECT_CLASS (g_type_class_peek (type));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	source = RB_GENERIC_PLAYER_SOURCE (parent_class->constructor (type, n_construct_properties, construct_properties));

	rb_generic_player_load_songs (source);

	return G_OBJECT (source);
}

static void 
rb_generic_player_source_dispose (GObject *object)
{
	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (object);

	if (priv->mount_path) {
		g_free (priv->mount_path);
		priv->mount_path = NULL;
	}
	
	G_OBJECT_CLASS (rb_generic_player_source_parent_class)->dispose (object);
}

RBRemovableMediaSource *
rb_generic_player_source_new (RBShell *shell, GnomeVFSVolume *volume)
{
	RBGenericPlayerSource *source;
	RhythmDBEntryType entry_type;

	g_assert (rb_generic_player_is_volume_player (volume));

	entry_type =  rhythmdb_entry_register_type ();

	source = RB_GENERIC_PLAYER_SOURCE (g_object_new (RB_TYPE_GENERIC_PLAYER_SOURCE,
					  "name", _("Portable Audio Player"),
					  "entry-type", entry_type,
					  "volume", volume,
					  "shell", shell,
					  NULL));

	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);

	return RB_REMOVABLE_MEDIA_SOURCE (source);
}

static void
rb_generic_player_load_songs (RBGenericPlayerSource *source)
{
	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
	GnomeVFSVolume *volume;
	RBShell *shell;
	RhythmDB *db;
	RhythmDBEntryType entry_type;

	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
	priv->mount_path = rb_generic_player_get_mount_path (volume);
	g_object_unref (G_OBJECT (volume));
	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
	g_object_get (G_OBJECT (shell), "db", &db, NULL);
	g_object_unref (G_OBJECT (shell));

	rhythmdb_add_uri_with_type (db, priv->mount_path, entry_type);
}

static gchar *
rb_generic_player_get_mount_path (GnomeVFSVolume *volume)
{
	gchar *path;
	gchar *uri;

	uri = gnome_vfs_volume_get_activation_uri (volume);
	path = g_filename_from_uri (uri, NULL, NULL);
	g_assert (path != NULL);
	g_free (uri);

	return path;
}


gboolean
rb_generic_player_is_volume_player (GnomeVFSVolume *volume)
{
	gboolean result = FALSE;
#ifdef HAVE_HAL
	gchar *udi = gnome_vfs_volume_get_hal_udi (volume);
	if (udi != NULL) {
		char *prop = NULL;
#ifdef HAVE_HAL_0_5
		LibHalContext *ctx = NULL;
		DBusConnection *conn = NULL;
		DBusError error;

		dbus_error_init (&error);
		ctx = libhal_ctx_new ();
		if (ctx == NULL) {
			g_print ("Error: %s\n", error.message);
			goto end;
		}
		conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
		if (conn == NULL) {
			g_print ("Error: %s\n", error.message);
			goto end;
		}
		libhal_ctx_set_dbus_connection (ctx, conn);

		if (libhal_device_query_capability (ctx, udi,
						    "portable_audio_player", &error))
			goto end;

		prop = libhal_device_get_property_string (ctx, udi,
							  "portable_audio_player.access_method", &error);
		if ((prop == NULL) || strcmp (prop, "storage") != 0)
			goto end;

		result = TRUE;
end:
		if (ctx) {
			libhal_ctx_shutdown (ctx, &error);
			libhal_ctx_free(ctx);
		}
		dbus_error_free (&error);
#elif HAVE_HAL_0_2
		LibHalContext *ctx = hal_initialize (NULL, FALSE);
		if (ctx == NULL)
			goto end;

		if (libhal_device_query_capability (ctx, udi,
						    "portable_audio_player"))
			goto end;

		prop = libhal_device_get_property_string (ctx, udi
							  "portable_audio_player.access_method");
		if ((prop == NULL) || strcmp (prop, "storage") != 0)
			goto end;

		result = TRUE;
end:
#endif
		g_free (prop);
		g_free (udi);
	}
#endif /* HAVE_HAL */
	return result;
}


static gboolean
impl_show_popup (RBSource *source)
{
	_rb_source_show_popup (RB_SOURCE (source), "/GenericPlayerSourcePopup");
	return TRUE;
}
